@@ -35,8 +35,8 @@ module Puppet
           }
 
       ...but for larger files, this attribute is more useful when combined with the
-      [template](https://docs.puppetlabs.com/references/latest/function.html#template)
-      or [file](https://docs.puppetlabs.com/references/latest/function.html#file)
+      [template](https://docs.puppetlabs.com/puppet/latest/reference/function.html#template)
+      or [file](https://docs.puppetlabs.com/puppet/latest/reference/function.html#file)
       function.
     EOT
 
